<?php

namespace Drupal\group_notify\Plugin;

use Drupal\gnode\Plugin\GroupContentEnabler\GroupNode;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class NotifyGroupNode extends GroupNode implements ContainerFactoryPluginInterface {

  /** @var array Email recipients */
  protected $recipients = [];

  /** @var \Drupal\Core\Mail\MailManagerInterface */
  protected $mailManager;

  /** @var \Drupal\Core\Extension\ModuleHandlerInterface */
  protected $moduleHandler;

  /** @var \Drupal\Core\Language\LanguageManagerInterface */
  protected $languageManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    MailManagerInterface $mail_manager,
    ModuleHandlerInterface $module_handler,
    LanguageManagerInterface $language_manager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->mailManager = $mail_manager;
    $this->moduleHandler = $module_handler;
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.mail'),
      $container->get('module_handler'),
      $container->get('language_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['notify'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Notify group members'),
      '#description' => $this->t('Send an email when group content is added.'),
      '#default_value' => $this->configuration['notify'] ?? FALSE
    ];

    $form['options'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Notification mode'),
      '#states' => ['visible' => [':input[name="notify"]' => ['checked' => TRUE]]]
    ];

    $form['options']['notify_mode'] = [
      '#type' => 'radios',
      '#default_value' => $this->configuration['notify_mode'] ?? 'enforce',
      '#options' => [
        'enforce' => $this->t('Enforced: Always send notification when a group node is created.'),
        'toggle' => $this->t('Toggled: Optionally send notification when a group node is created.')
      ]
    ];

    if ($this->moduleHandler->moduleExists('comment')) {
      $form['options']['comments'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Enable comment notifications'),
        '#description' => $this->t('Notify members of a group when a new comment is posted to a group node.'),
        '#default_value' => $this->configuration['comments'] ?? FALSE
      ];
    }

    return $form;
  }

  /**
   * Send notification to all recipients.
   *
   * @param array $params
   *   An array of mail parameters.
   */
  public function sendNotification(array $params) {
    foreach ($this->recipients as $recipient) {
      $result = $this->mailManager->mail(
        'group_notify',
        $params['mail_key'],
        $recipient['email'],
        $recipient['langcode'] ?: $this
          ->languageManager
          ->getCurrentLanguage()
          ->getId(),
        $params
      );

      // @todo Do something with $result['result'] perhaps?
    }
  }

  /**
   * Return list of recipients
   *
   * @return array
   */
  public function getRecipients() {
    return $this->recipients;
  }

  /**
   * Are there any recipients?
   *
   * @return bool
   */
  public function hasRecipients() {
    return sizeof($this->recipients) > 0;
  }

  /**
   * Add recipient for notifcation.
   *
   * @param array $recipient
   *   Array needs to include email address and langcode.
   *
   * @return \Drupal\group_notify\Plugin\NotifyGroupNode
   */
  public function addRecipient(array $recipient) {
    $this->recipients[] = $recipient;
    return $this;
  }

  /**
   * Are comment notifcations enabled?
   *
   * @return bool
   */
  public function isNotifyCommentEnabled() {
    $config = $this->getConfiguration();
    return $config['comments'] ?? FALSE;
  }

  /**
   * Are notifications toggled?
   *
   * @return bool
   */
  public function isNotifyToggled() {
    $config = $this->getConfiguration();
    return $config['notify_mode'] === 'toggle';
  }

  /**
   * Are notifications enforced?
   *
   * @return bool
   */
  public function isNotifyEnforced() {
    $config = $this->getConfiguration();
    return $config['notify_mode'] === 'enforce';
  }

  /**
   * Are notifications enabled?
   *
   * @return bool
   */
  public function isNotifyEnabled() {
    $config = $this->getConfiguration();
    return $config['notify'] ?? FALSE;
  }

}

